; ---------------------------------------------------------------------------
; crt0.s
; ---------------------------------------------------------------------------
;
; Startup code for cc65 (Proxy SBC v2)

.export _init, _exit
.import _main, _brk_isr, _irq_isr, _nmi_isr

.export __STARTUP__ : absolute = 1		; Mark as startup
.import __RAM_START__, __RAM_SIZE__		; Linker generated

.import copydata, zerobss, initlib, donelib

.include "zeropage.inc"

.PC02		; Force 65C02 assembly mode

; ---------------------------------------------------------------------------
; Place the startup code in a special segment

.segment "STARTUP"

; ---------------------------------------------------------------------------
; Set up the Stack and some Flags

_init:
	LDX #$FF
	TXS			; Initialize stack pointer to 0xFF
	CLD			; Disable BCD Mode
	SEI			; Disable Interrupts

; ---------------------------------------------------------------------------
; Set CC65 argument stack pointer

	LDA #<(__RAM_START__ + __RAM_SIZE__)
	LDX #>(__RAM_START__ + __RAM_SIZE__)
	STA sp
	STX sp+1

; ---------------------------------------------------------------------------
; Initialize memory storage

	JSR zerobss		; Clear BSS segment
	;JSR copydata	; Initialize DATA segment, any reason for this? since Data will be in RAM anyways...
	JSR initlib		; Run constructors

; ---------------------------------------------------------------------------
; Set up the Interrupt Vectors in ZP, as the ROM redirects all Interrupts to these
	LDA #>_brk_isr
	LDX #<_brk_isr
	STA $00
	STX $01
	
	LDA #>_irq_isr
	LDX #<_irq_isr
	STA $02
	STX $03
	
	LDA #>_nmi_isr
	LDX #<_nmi_isr
	STA $04
	STX $05
	

; ---------------------------------------------------------------------------
; Call main()

	JSR _main

; ---------------------------------------------------------------------------
; Back from main (this is also the _exit entry)

_exit:
	JSR donelib		; Run destructors
	JMP ($FFF8)		; Indirectly Jump to the Termination Function in ROM